@@ -1,6 +1,6 @@
 /*
 
-    Copyright (C) 2011,2013,2015,2022 Alois Schloegl <alois.schloegl@ist.ac.at>
+    Copyright (C) 2011,2013,2015,2022,2024,2025 Alois Schloegl <alois.schloegl@ist.ac.at>
     This file is part of the "BioSig for C/C++" repository
     (biosig4c++) at http://biosig.sf.net/
 
@@ -34,6 +34,8 @@ extern int VERBOSE_LEVEL; 	// used for debugging
   #endif
 #endif
 
+static const double EPS  = 0x1p-52;  /* about 2.220e-16 */
+
 double getDouble(const mxArray *pm, size_t idx) {
 	size_t n = mxGetNumberOfElements(pm);
 	if (n == 0)   return(NAN);
@@ -89,14 +91,15 @@ void mexFunction(
 	time_t 		T0;
 	char 		*FileName;
 	char 		tmpstr[128];
-	int 		status;
 	int		CHAN = 0;
 	double		*ChanList=NULL;
 	int		NS = -1;
-	char		FlagOverflowDetection = 1, FlagUCAL = 0;
+	char		FlagOverflowDetection = -1, FlagUCAL = -1;
 	void 		*data = NULL;
 	mxArray *p = NULL, *p1 = NULL, *p2 = NULL;
 	uint16_t gdftyp = 0;
+	int		istatus;
+	double		*status;
 
 #ifdef CHOLMOD_H
 	cholmod_sparse RR,*rr=NULL;
@@ -121,6 +124,11 @@ void mexFunction(
 	VERBOSE_LEVEL=0;	// set debug level to minimum
 #endif
 
+	if (nlhs > 0) {
+		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
+		status = mxGetPr(plhs[0]);
+		*status = 0.0;
+	}
 /*
  	improve checks for input arguments
 */
@@ -169,11 +177,13 @@ void mexFunction(
 		case mxCELL_CLASS:
 		case mxSTRUCT_CLASS:
 		default:
+			*status = -1.0;
 			mexErrMsgTxt("mexSSAVE(HDR,data) failed datatype of data is not supported\n");
 			return;
 		}
 	}
 	else {
+		*status = -2.0;
 		mexErrMsgTxt("mexSSAVE(HDR,data) failed because HDR and data, are not a struct and numeric, resp.\n");
 		return;
 	}
@@ -196,14 +206,13 @@ void mexFunction(
 				FlagUCAL = 0;
 		}
 		else {
+			*status = -3.0;
 			mexPrintf("%s: argument #%i is invalid.",__FILE__, k+1);
 			mexErrMsgTxt("mexSOPEN fails because of unknown parameter\n");
 		}
 	}
 
 	/***** SET INPUT ARGUMENTS *****/
-	hdr->FLAG.OVERFLOWDETECTION = FlagOverflowDetection;
-	hdr->FLAG.UCAL = FlagUCAL;
 #ifdef CHOLMOD_H
 	hdr->FLAG.ROW_BASED_CHANNELS = (rr!=NULL);
 #else
@@ -213,6 +222,21 @@ void mexFunction(
 	/***** CHECK INPUT HDR STUCTURE CONVERT TO libbiosig hdr *****/
 	if (VERBOSE_LEVEL>7) mexPrintf("110: input arguments checked\n");
 
+	if ( (p = mxGetField(prhs[0], 0, "FLAG") ) != NULL ) {
+		if (FlagUCAL==-1)
+			if ( (p1 = mxGetField(p, 0, "UCAL") ) != NULL ) {
+				FlagUCAL = (fabs(getDouble(p1, 0)) < EPS);
+			}
+		if (FlagOverflowDetection==-1)
+			if ( (p1 = mxGetField(p, 0, "OVERFLOWDETECTION") ) != NULL ) {
+				FlagOverflowDetection = (fabs(getDouble(p1, 0)) < EPS);
+			}
+	}
+	if (FlagUCAL==-1) FlagUCAL=0;
+	hdr->FLAG.UCAL = FlagUCAL;
+	if (FlagOverflowDetection==-1) FlagOverflowDetection=1;
+	hdr->FLAG.OVERFLOWDETECTION = FlagOverflowDetection;
+
 	if ( (p = mxGetField(prhs[0], 0, "TYPE") ) != NULL ) {
 		mxGetString(p,tmpstr,sizeof(tmpstr));
 		hdr->TYPE 	= GetFileTypeFromString(tmpstr);
@@ -228,7 +252,7 @@ void mexFunction(
 			break;
 		}
 	}
-	if ( (p = mxGetField(prhs[0], 0, "T0") ) != NULL ) 		hdr->T0 	= (gdf_time)getDouble(p, 0);
+	if ( (p = mxGetField(prhs[0], 0, "T0") ) != NULL ) 		hdr->T0 = (gdf_time)ldexp(getDouble(p, 0),32);
 	if ( (p = mxGetField(prhs[0], 0, "tzmin") ) != NULL )
 		hdr->tzmin 	= (int16_t)getDouble(p, 0);
 	else {
@@ -499,7 +523,7 @@ void mexFunction(
 		if ( (p1 = mxGetField(p, 0, "POS") ) != NULL ) {
 			size_t n = mxGetNumberOfElements(p1);
 			for (k = 0; k < n; k++)
-				hdr->EVENT.POS[k] = (uint32_t)getDouble(p1,k);
+				hdr->EVENT.POS[k] = (uint32_t)getDouble(p1,k) - 1;   // convert from 1-based indexing to 0-based indexing
 		}
 		if ( (p1 = mxGetField(p, 0, "TYP") ) != NULL ) {
 			size_t n = mxGetNumberOfElements(p1);
@@ -548,13 +572,24 @@ void mexFunction(
 	}
 
 	hdr = sopen(FileName, "w", hdr);
-	if (serror2(hdr)) mexErrMsgTxt("mexSSAVE: sopen failed \n");
+	istatus = serror2(hdr);
+	if (istatus) {
+		mexErrMsgTxt("mexSSAVE: sopen failed \n");
+		*status = istatus;
+	}
 
 	swrite((biosig_data_type*)data, hdr->NRec, hdr);
-	if (serror2(hdr)) mexErrMsgTxt("mexSSAVE: swrite failed \n");
+	istatus = serror2(hdr);
+	if (istatus) {
+		mexErrMsgTxt("mexSSAVE: swrite failed \n");
+		*status = istatus;
+	}
 
 	destructHDR(hdr);
-	if (serror2(hdr)) mexErrMsgTxt("mexSSAVE: sclose failed \n");
-
+	istatus = serror2(hdr);
+	if (istatus) {
+		mexErrMsgTxt("mexSSAVE: sclose failed \n");
+		*status = istatus;
+	}
 };
 
