@@ -213,6 +213,12 @@ EXTERN_C void sopen_atf_read(HDRTYPE* hdr) {
 			hdr->EVENT.TYP[k] = 0x7ffe;
 			hdr->EVENT.POS[k] = (k+1)*hdr->SPR;
 		}
+		if (hdr->EVENT.DUR!=NULL && hdr->EVENT.CHN!=NULL) {
+			for (size_t k=0; k < hdr->EVENT.N ; k++) {
+				hdr->EVENT.DUR[k]=0;	// duration
+				hdr->EVENT.CHN[k]=0;	// all channels
+			}
+		}
 	}
 
 	/* reshape data */
@@ -290,6 +296,9 @@ EXTERN_C void sopen_abf_read(HDRTYPE* hdr) {
 		switch (lei16p(hdr->AS.Header + offsetof(struct ABFFileHeader, nDataFormat))) {
 		case 0: gdftyp = 3; break;
 		case 1: gdftyp = 16; break;
+		default:
+			biosigERROR(hdr, B4C_DATATYPE_UNSUPPORTED, "ABF datatype/gdftyp unknown/unsupported (neither int16 nor float32)");
+			break;
 		}
 
 	if (VERBOSE_LEVEL>7) fprintf(stderr,"%s (line %i)\n",__FILE__,__LINE__);
@@ -543,6 +552,12 @@ EXTERN_C void sopen_abf_read(HDRTYPE* hdr) {
 			hdr->EVENT.TYP[k] = 0x7ffe;
 			hdr->EVENT.POS[k] = (k+1)*spr;
 		}
+		if (hdr->EVENT.DUR!=NULL && hdr->EVENT.CHN!=NULL) {
+			for (k=0; k < n1; k++) {
+				hdr->EVENT.DUR[k]=0;	// duration
+				hdr->EVENT.CHN[k]=0;	// all channels
+			}
+		}
 		/* add tags */
 		hdr->AS.auxBUF = realloc(hdr->AS.auxBUF, n2 * sizeof(struct ABFTag));
 		ifseek(hdr, lei32p(hdr->AS.Header + offsetof(struct ABFFileHeader, lTagSectionPtr))*ABF_BLOCKSIZE, SEEK_SET);
@@ -613,6 +628,7 @@ EXTERN_C void sopen_abf2_read(HDRTYPE* hdr) {
 		hdr->SPR = 1;
 		uint16_t gdftyp = 3;
 
+		unsigned numSweeps = leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, uActualEpisodes));
 		if (VERBOSE_LEVEL>7) {
 			fprintf(stdout,"\nuFileInfoSize:\t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, uFileInfoSize)));
 			fprintf(stdout,"uActualEpisodes:\t%i\n",leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, uActualEpisodes)));
@@ -709,6 +725,9 @@ EXTERN_C void sopen_abf2_read(HDRTYPE* hdr) {
 		switch(leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DataSection.uBytes))) {
 		case 2: gdftyp=3; break;
 		case 4: gdftyp=16; break;
+		default:
+			biosigERROR(hdr, B4C_DATATYPE_UNSUPPORTED, "ABF2 datatype/gdftyp unknown/unsupported (neither int16 nor float32)");
+			break;
 		}
 
 		struct ABF_Section S;
@@ -731,7 +750,7 @@ EXTERN_C void sopen_abf2_read(HDRTYPE* hdr) {
 			fprintf(stdout,"Warning ABF2 v%4.2f: nOperationMode=%d is very experimental - double check the data you get.\n", hdr->VERSION, nOperationMode);
 			break;
 		default:
-			biosigERROR(hdr, B4C_DATATYPE_UNSUPPORTED, "ABF2 nOperationMode=%d unknown and unsupported");
+			biosigERROR(hdr, B4C_DATATYPE_UNSUPPORTED, "ABF2 nOperationMode unknown and unsupported");
 			break;
 		}
 
@@ -792,6 +811,7 @@ EXTERN_C void sopen_abf2_read(HDRTYPE* hdr) {
 			strncpy(hc->Label, (char*)hdr->AS.Header + offsetof(struct ABFFileHeader, sADCChannelName) + k*ABF_ADCNAMELEN, min(ABF_ADCNAMELEN,MAX_LENGTH_LABEL));
 			hc->Label[ABF_ADCNAMELEN] = 0;
 /*
+			// TODO: where to find information about PhysDim and Label in ABF2 file ? Currently, this is hardcoded below
 			char units[ABF_ADCUNITLEN+1]; {
 				memcpy(units, (char*)hdr->AS.Header + offsetof(struct ABFFileHeader, sADCUnits) + k*ABF_ADCUNITLEN, ABF_ADCUNITLEN);
 				units[ABF_ADCUNITLEN] = 0;
@@ -800,6 +820,9 @@ EXTERN_C void sopen_abf2_read(HDRTYPE* hdr) {
 			}
 */
 
+			hc->PhysDimCode = PhysDimCode("V");
+			hc->Cal = 0.001;
+			hc->Off = 0.0;
 			double PhysMax = fADCRange;
 			double DigMax  = lADCResolution;
 
@@ -807,16 +830,16 @@ EXTERN_C void sopen_abf2_read(HDRTYPE* hdr) {
 			// https://support.moleculardevices.com/s/article/Convert-data-file-from-another-program-to-an-ABF-file-so-that-it-can-be-read-by-pCLAMP
 			// https://github.com/yamad/libabf/blob/master/src/AxAbfFio32/abfheader.cpp -> GetADCtoUUFactors
 
-			{
-			double iCal = lef32p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, fInstrumentScaleFactor));
-			iCal *= lef32p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, fADCProgrammableGain));
-			if (lei16p(hdr->AS.Header + offsetof(struct ABFFileHeader, nSignalType) + 2 * k) != 0)
-				iCal *= lef32p(hdr->AS.Header + offsetof(struct ABFFileHeader, fSignalGain) + 4 * k);
-			if (!lei16p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, nTelegraphEnable)))
-				iCal *= lef32p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, fTelegraphAdditGain));
-			if (hc->Cal == 0.0) hc->Cal = 1.0;
-			hc->Cal = fADCRange / (iCal * lADCResolution);
-			hc->Off = lef32p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, fInstrumentOffset));
+			if (gdftyp==3) {
+				double iCal = lef32p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, fInstrumentScaleFactor));
+				iCal *= lef32p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, fADCProgrammableGain));
+				if (lei16p(hdr->AS.Header + offsetof(struct ABFFileHeader, nSignalType) + 2 * k) != 0)
+					iCal *= lef32p(hdr->AS.Header + offsetof(struct ABFFileHeader, fSignalGain) + 4 * k);
+				if (!lei16p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, nTelegraphEnable)))
+					iCal *= lef32p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, fTelegraphAdditGain));
+				if (hc->Cal == 0.0) hc->Cal = 1.0;
+				hc->Cal = fADCRange / (iCal * lADCResolution);
+				hc->Off = lef32p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, fInstrumentOffset));
 			}
 
 			// hc->Off      = lef32p(hdr->AS.auxBUF + S.uBytes*k + offsetof(struct ABF_ADCInfo, fInstrumentOffset));
@@ -880,5 +903,22 @@ EXTERN_C void sopen_abf2_read(HDRTYPE* hdr) {
 		// beginning of data block
 		hdr->HeadLen = leu32p(hdr->AS.Header + offsetof(struct ABF_FileInfo, DataSection.uBlockIndex)) * 512;
 
+
+		/* set up event table  */
+		if (numSweeps > 1) {
+			reallocEventTable(hdr, numSweeps - 1);
+			hdr->EVENT.N = numSweeps - 1;
+			hdr->EVENT.SampleRate = hdr->SampleRate;
+			for (size_t k=0; k < hdr->EVENT.N ; k++) {
+				hdr->EVENT.TYP[k] = 0x7ffe;
+				hdr->EVENT.POS[k] = (k+1)*((uint64_t)hdr->SPR * hdr->NRec) / numSweeps;
+			}
+			if (hdr->EVENT.DUR!=NULL && hdr->EVENT.CHN!=NULL) {
+				for (size_t k=0; k < hdr->EVENT.N ; k++) {
+					hdr->EVENT.DUR[k] = 0;	// duration
+					hdr->EVENT.CHN[k] = 0;	// all channels
+				}
+			}
+		}
 }
 
