@@ -394,7 +394,7 @@ if (VERBOSE_LEVEL > 7) fprintf(stdout,"%s (line %i) NS=%i nCol=%i\n", __FILE__,
 				size_t inlen      = beu32p((uint8_t*)(ValLabel[ns])-4);
 				char *outbuf      = hc->Label;
 				size_t outlen     = MAX_LENGTH_LABEL+1;
-#if  defined(_ICONV_H) || defined (_LIBICONV_H)
+#if  defined(_ICONV_H) || defined (_LIBICONV_H) || defined(_ICONV_H_)
 				iconv_t ICONV = iconv_open("UTF-8","UCS-2BE");
 				size_t reticonv = iconv(ICONV, &inbuf, &inlen, &outbuf, &outlen);
 				iconv_close(ICONV);
