@@ -122,15 +122,12 @@ gdf_time heka2gdftime(double t) {
 void sopen_heka(HDRTYPE* hdr, FILE *itx) {
 	size_t count = hdr->HeadLen;
 
-	if (hdr->TYPE==HEKA && hdr->VERSION > 1) {
-
-		int32_t Levels=0;
+	if (hdr->TYPE==HEKA) {
+		uint32_t Levels=0;
 		uint16_t k;
-		//int32_t *Sizes=NULL;
-		int32_t Counts[5], counts[5]; //, Sizes[5];
+		int32_t Counts[5], counts[5];
 		memset(Counts,0,20);
 		memset(counts,0,20);
-		//memset(Sizes,0,20);
 		uint32_t StartOfData=0,StartOfPulse=0;
 
 		union {
@@ -143,7 +140,7 @@ void sopen_heka(HDRTYPE* hdr, FILE *itx) {
 			} Rec;
 			int32_t all[5];
 		} Sizes;
-
+		memset(&Sizes,0,20);
 
     		// HEKA PatchMaster file format
 
@@ -155,35 +152,49 @@ void sopen_heka(HDRTYPE* hdr, FILE *itx) {
 			count += ifread(hdr->AS.Header+count, 1, 1024-count, hdr);
 		hdr->HeadLen = count;
 
-		hdr->FILE.LittleEndian = *(uint8_t*)(hdr->AS.Header+52) > 0;
 		char SWAP = ( hdr->FILE.LittleEndian && (__BYTE_ORDER == __BIG_ENDIAN))  \
 			 || (!hdr->FILE.LittleEndian && (__BYTE_ORDER == __LITTLE_ENDIAN));
 
 		if (SWAP) {
-			biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "Heka/Patchmaster format requires data swapping - this is not supported yet.");
-			return;
+			// FIXME: support HEKA files that require swapping, it might be releated to the version of the HEKA format
+			biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "Heka/Patchmaster format: this is not fully supported yet.");
 		}
-		SWAP = 0;  // might be useful for compile time optimization
 
 		/* get file size and read whole file */
 		count += ifread(hdr->AS.Header+count, 1, FileBuf.st_size - count, hdr);
 
 if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) %s(...): %i bytes read\n",__FILE__,__LINE__,__func__, (int)count);
 
-		// double oTime;
-		uint32_t nItems;
-		if (hdr->FILE.LittleEndian) {
-			// oTime  = lef64p(hdr->AS.Header+40);	// not used
-			nItems = leu32p(hdr->AS.Header+48);
-		}
-		else {
-			// oTime  = bef64p(hdr->AS.Header+40);	// not used
-			nItems = beu32p(hdr->AS.Header+48);
+		// double oTime = lef64p(hdr->AS.Header+40);	// not used, do not forget to swap if needed
+
+		uint32_t nItems = *(uint32_t*)(hdr->AS.Header+48);
+		Levels = *(int32_t*)(hdr->AS.Header+4);
+		if (SWAP) {
+			nItems = bswap_32(nItems);
+			Levels = bswap_32(Levels);
 		}
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) %s(...): nItems=%i\n",__FILE__,__LINE__,__func__, nItems);
+if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) %s(...): nItems=%d Levels=%d\n",__FILE__,__LINE__,__func__, nItems,Levels);
 
-		if (hdr->VERSION == 1) {
+		if (Levels>5) {
+			biosigERROR(hdr, B4C_FORMAT_UNSUPPORTED, "Heka/Patchmaster format with more than 5 levels not supported");
+			return;
+		}
+		memcpy(Sizes.all, hdr->AS.Header+8, sizeof(uint32_t)*Levels);
+		if (SWAP) {
+			for (int l=0; l < Levels; l++)
+				Sizes.all[l] = bswap_32(Sizes.all[l]);
+		}
+
+if (VERBOSE_LEVEL>6)
+		for (int l=0; l < Levels; l++)
+			 fprintf(stdout,"%s (line %i) %s(...): Sizes[level=%d]=%d\n",__FILE__,__LINE__,__func__, l, Sizes.all[l]);
+
+
+		if (hdr->VERSION == 0) {
+			StartOfPulse = 8 + 4 * Levels;
+		}
+		else if (hdr->VERSION == 1) {
 			Sizes.Rec.Root   = 544;
 			Sizes.Rec.Group  = 128;
 			Sizes.Rec.Series = 1120;
@@ -272,7 +283,7 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) %s(...): \n",__FILE__,__LINE__
 		// read K1
 		if (SWAP) {
 			K1 		= bswap_32(*(uint32_t*)(hdr->AS.Header + StartOfPulse + Sizes.Rec.Root));
-			hdr->VERSION 	= bswap_32(*(uint32_t*)(hdr->AS.Header + StartOfPulse));
+			hdr->VERSION	= bswap_32(*(uint32_t*)(hdr->AS.Header + StartOfPulse));
 			union {
 				double f64;
 				uint64_t u64;
@@ -292,8 +303,6 @@ if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) %s(...): \n",__FILE__,__LINE__
 		double *DT = NULL; 	// list of sampling intervals per channel
 		hdr->SPR = 0;
 
-if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %i) %s(...): %p\n",__FILE__,__LINE__,__func__,hdr->EVENT.CodeDesc);
-
 		/*******************************************************************************************************
 			HEKA: read structural information
  		 *******************************************************************************************************/
@@ -455,6 +464,7 @@ if (VERBOSE_LEVEL>7) fprintf(stdout, "%s (line %i): %i %i %i %i %i %g %g 0x%x xU
 							//c.f64 = PhysMax; c.u64 = bswap_64(c.u64); PhysMax = c.f64;
 							//c.f64 = PhysMin; c.u64 = bswap_64(c.u64); PhysMin = c.f64;
 							c.f64 = Toffset; c.u64 = bswap_64(c.u64); Toffset = c.f64;
+							c.f64 = DataScaler; c.u64 = bswap_64(c.u64); DataScaler = c.f64;
  						}
 
 						if (YOffset != 0.0) 
