@@ -252,9 +252,9 @@ int sopen_rhs2000_read(HDRTYPE* hdr) {
 		if (VERBOSE_LEVEL>7) fprintf(stdout,"%s (line %u); %s(...) [%u]\n",__FILE__,__LINE__,__func__,hdr->HeadLen);
 
 		// 8 bytes
-		float minor = leu16p(hdr->AS.Header+6);
-		minor      *= (minor < 10) ? 0.1 : 0.01;
-		hdr->VERSION = leu16p(hdr->AS.Header+4) + minor;
+		int16_t major = leu16p(hdr->AS.Header+4);
+		int16_t minor = leu16p(hdr->AS.Header+6);
+		hdr->VERSION = major + minor * (minor < 10 ? 0.1 : 0.01);
 
 		// +38 = 46 bytes
 		hdr->NS = 1;
@@ -297,7 +297,7 @@ int sopen_rhs2000_read(HDRTYPE* hdr) {
 		pos += 2;
 		uint16_t NS = 0;
 
-		hdr->SPR = 128;
+		hdr->SPR = (major == 1 ? 60 : 128);
 		size_t bi = (0+4)*hdr->SPR;
 
 		// read all signal groups
