@@ -35,11 +35,13 @@ function [results, option] = microstimfit(data, Fs, evtpos, varargin)
 %      optioon.baseFlag: 0 (default): mean (average) of baseline
 %                        1 : median of baseline
 %      option.fitFlag	0 [default]: no fitting
-%			1: fit decay with monoexponential model 'a*exp(-x/tau)+offset' (from peakTime to t2)
-%			2: fit decay with biexponential model 'a1*exp(-x/tau1)+a2*exp(-x/tau2) ' (from peakTime to t2)
-%			3: fit decay with biexponential model 'a1*exp(-x/tau1)+a2*exp(-x/tau2)+offset' (from peakTime to t2)
-%      option.thres	threshold value for AP (in relative units or unit voltage or current per sample interval), and 
-%      option.thresFlag	controls the threshold criterion (0 = relative to max slope, 1 = absolute). 
+%			1: fit decay with monoexponential model 'a*exp(-x/tau)+offset'
+%			2: fit decay with biexponential model 'a1*exp(-x/tau1)+a2*exp(-x/tau2) '
+%			3: fit decay with biexponential model 'a1*exp(-x/tau1)+a2*exp(-x/tau2)+offset'
+%			5: biexponential with delay: 'a0*exp(delay-x/tau1)-exp(delay-x/tau2)+offset'
+%      option.thres	threshold value for AP (in relative units or unit voltage or current per sample interval), and
+%      option.thresFlag	controls the threshold criterion (0 = relative to max slope, 1 = absolute).
+%      option.fitBegin	start of fitting window (only for fitFlag=5)
 %      option.fitEnd	data length for fitting, starting from the peak time 
 %
 %  Output arguments:
@@ -59,7 +61,7 @@ function [results, option] = microstimfit(data, Fs, evtpos, varargin)
 %     https://pub.ist.ac.at/~schloegl/publications/GuzmanEtAl2014.fninf-08-00016.pdf
 
 
-% Copyright (C) 2017-2019,2022 Alois Schlögl, IST Austria <alois.schloegl@ist.ac.at>
+% Copyright (C) 2017-2019,2022,2023 Alois Schlögl, IST Austria <alois.schloegl@ist.ac.at>
 
 % TODO: 
 % smoothing window length 
@@ -122,8 +124,9 @@ default.t2=round(+Fs*100e-3);
 default.baseEnd=round(-Fs*10e-3);
 default.peakBeg=round(-Fs*10e-3);
 default.peakEnd=round(+Fs*20e-3);
+default.fitBegin=round(+Fs*0e-3);
 default.fitEnd=round(+Fs*50e-3);
-default.meanN=round(50e-6*Fs);	% see [1], Fig. 3-C
+default.meanN=max([1,round(50e-6*Fs)]);	% see [1], Fig. 3-C
 default.dir=0;
 default.plotFlag=0;
 default.baseFlag=0;
@@ -186,7 +189,8 @@ N = length(evtpos);
 results.label = {'tix','baseline','baseSD','-','peak','tPeak', ...
 		 't20Real', 't80Real', 't50AInt', 't50AReal', 't50BInt', 't50BReal', 't0Real', ...
 		 'tMaxSlopeRiseReal', 'maxSlopeRiseReal', 'tThreshold', 'yThreshold', ...
-		 'tMaxSlopeDecayReal', 'maxSlopeDecayReal', ...
+		 'tMaximum', 'Maximum', ...
+		 'tMinimum', 'Minimum', ...
 		};
 
 numFixLabels = length(results.label);
@@ -214,6 +218,12 @@ elseif (option.fitFlag==3)
 	option.fitfun = @(p, x) (p(1) * exp (-x*p(3)) + p(2) * exp (-x*p(4)) + p(5));	% A1 * exp(-t*invTau1) + A2 * exp(-t*invTau2) + offset
 	results.fitResults=repmat(NaN,N,5);
 	results.weightedTau=repmat(NaN,N,1);
+elseif (option.fitFlag==5)
+	warning('fitting model "biexponential with delay" (fitFlag==5) is experimental')
+	results.label(end+[1:5]) = {'A0','tau1 [s]', 'tau2 [s]','delay', 'offset'};
+	option.fitfun = @(p, x) (p(1) * exp (p(4)-x*p(2) - exp (p(4)-x*p(3))) + p(5));	% A0 * (exp(delay-t*invTau1) - exp(delay-t*invTau2)) + offset
+	results.fitResults=repmat(NaN,N,5);
+	results.weightedTau=repmat(NaN,N,1);
 elseif (option.fitFlag==0)
 	;
 else
@@ -304,8 +314,12 @@ for k=1:N;
 
 	peakRegion = sdata( max(option.peakBegin+ix,1) : min(ix+option.peakEnd,size(data,1)) ) - base;
 
-	if option.dir && any(xor(peak < 0, option.dir < 0))
+	if option.dir && xor((peak(k) < 0), (option.dir < 0)),
+		[k,option.dir, peak(k) < 0]
 		warning('microStimfit: peak has wrong direction  - this is strange'); 
+		tt=-1000:1000;
+		plot(tt/Fs,sdata(tt+ix))
+		pause
 	end
 
 	peakCor     = abs(peak(k));
@@ -317,25 +331,29 @@ for k=1:N;
 	t50AInt = find( diff(peakRegion2 >  0.5 * peakCor) > 0); t50AInt(t50AInt > tPeak2(k))=[];
 	t50BInt = find( diff(peakRegion2 >= 0.5 * peakCor) < 0); t50BInt(t50BInt < tPeak2(k))=[];
 
-	t20Real = ix + option.peakBegin - 1; 
-	t80Real = ix + option.peakBegin - 1;
-	t50AReal= ix + option.peakBegin - 1;
-	t50BReal= ix + option.peakEnd   - 1;
-
+	t20Real = ix + option.peakBegin - 1;
 	if (length(t20Int)>0)
 		t20Real  = t20Int  + (0.2*peakCor-peakRegion2(t20Int))./peakRegion3(t20Int) + ix + option.peakBegin-1; 
 	end;
+	t80Real = ix + option.peakBegin - 1;
 	if (length(t80Int)>0)
 		t80Real  = t80Int  + (0.8*peakCor-peakRegion2(t80Int))./peakRegion3(t80Int) + ix + option.peakBegin-1;
 	end;
+	t50AReal= ix + option.peakBegin - 1;
 	if (length(t50AInt)>0)
 		t50AReal = t50AInt + (0.5*peakCor-peakRegion2(t50AInt))./peakRegion3(t50AInt) + ix + option.peakBegin-1;
 	end;
+	t50BReal= ix + option.peakEnd   - 1;
 	if (length(t50BInt)>0)
 		t50BReal = t50BInt + (0.5*peakCor-peakRegion2(t50BInt))./peakRegion3(t50BInt) + ix + option.peakBegin-1;
 	end;
 	t0Real = mean(t20Real) - (mean(t80Real)-mean(t20Real))/3;
 
+	[Maximum,tMaximum] = max(peakRegion);
+	tMaximum = mean(tMaximum) + ix + option.peakBegin - 1;
+	[Minimum,tMinimum] = min(peakRegion);
+	tMinimum = mean(tMinimum) + ix + option.peakBegin - 1;
+
 	maxSlopeRiseReal  = max(peakRegion3);
 	tMaxSlopeRiseReal = find(peakRegion3==maxSlopeRiseReal) + ix + option.peakBegin - 0.5;
 	maxSlopeDecayReal = min(peakRegion3);
@@ -354,6 +372,10 @@ for k=1:N;
 	results.data(k,15) = mean(maxSlopeRiseReal);
 	results.data(k,18) = mean(tMaxSlopeDecayReal);
 	results.data(k,19) = mean(maxSlopeDecayReal);
+	results.data(k,20) = mean(tMaximum);
+	results.data(k,21) = mean(Maximum);
+	results.data(k,22) = mean(tMinimum);
+	results.data(k,23) = mean(Minimum);
 
 	% peakRegion4
 	if option.thresFlag>0
@@ -451,6 +473,30 @@ for k=1:N;
 			warning('fitting failed; optimization toolbox or optim package missing')
 			fitResult=[];
 		end
+
+	elseif (option.fitFlag==5)
+		% results.label(end+[1:3]) = {'A0', 'tau1 [s]', 'tau2 [s]', 'delay [s]', offset};
+		%         fitfun = @(p, x) (p(1) * exp (p(4)-x*p(2) - exp (p(4)-x*p(3)) + p(5));	% A0 * (exp(delay-t*invTau1) - exp(delay-t*invTau2)) + offset
+
+		fitResult = [];
+		fExp      = option.fitfun;
+		invtau1   = Fs/(mean(t50BReal) - mean(t50AReal));
+		pInit = [peak(k), Fs/(mean(t50AReal)), Fs/(mean(t50BReal)), 0, base];
+		LB = [-inf, 0, 0, 0, -inf];
+		UB = [+inf, +inf, +inf, +inf, +inf];
+
+		try
+			% A0 * (exp(delay-t*invTau1) - exp(delay-t*invTau2)) + offset
+			fitfun = @(p, x) (p(1) * (exp (p(4)-x*p(2)) - exp(p(4)-x*p(3))) + p(5));
+			t = [0:(default.option.fitEnd-default.option.fitBegin)]'/Fs;
+			decay = data(default.option.fitBegin(k) + t);
+			[fitResult, RESNORM, RESIDUAL, EXITFLAG, OUTPUT, LAMBDA, JACOBIAN] = lsqcurvefit (option.fitfun, pInit', t, decay, LB, UB)
+			results.fitResults(k,1:5) = [fitResult(1), 1/fitResult(2), 1/fitResult(3), fitResult(4), fitResult(5)];
+			results.weightedTau(k,1)  = 1./fitResult(2:3);
+		catch
+			warning('fitting failed; optimization toolbox or optim package missing')
+			fitResult=[];
+		end
 	end
 
 	if (option.plotFlag),
@@ -482,7 +528,7 @@ for k=1:N;
 		set(h(end-2),'linewidth',lineWidth);
 		set(h(end-1),'linewidth',lineWidth);
 		set(h(end),'linewidth',lineWidth);
-		title(sprintf('%i/%i',k,N))
+		title(sprintf('%i / %i (t = %g s)',k,N,evtpos(k)/Fs))
 		
 		if any(option.fitFlag==[1:3]) && ~isempty(fitResult)
 			hold on;
@@ -492,10 +538,10 @@ for k=1:N;
 			%plot(x/Fs,fitResult(x),'c','linewidth',4);
 			hold off
 			drawnow
-			if option.plotFlag > 0,
-				fprintf(1,'Press any key');
-				pause;
-			end
+		end
+		if option.plotFlag > 0,
+			fprintf(1,'Press any key');
+			pause;
 		end
 	end
 end
@@ -514,5 +560,7 @@ results.MaxSlopeTime   = diff(results.data(:,[1,14]),[],2)/Fs;	% for backwards c
 results.tMaxRiseSlope  = diff(results.data(:,[1,14]),[],2)/Fs;	% same  .MaxSlopeTime
 results.tMaxDecaySlope = diff(results.data(:,[1,18]),[],2)/Fs;
 results.thresholdTime = diff(results.data(:,[1,16]),[],2)/Fs;
+results.MaxTime = (results.data(:,20)-evtpos)/Fs;
+results.MinTime = (results.data(:,22)-evtpos)/Fs;
 
 
